(* ::Package:: *)

(* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Model file: The Scalar Singlet Dark Matter model

Model parameters:

    mDM     -   DM mass
    ls      -   coupling to the Higgs
    QCDcase -   assumption choice regarding the QCD phase transition

*************** suggested accuracy settings  ***************
plotPoints  =   200;
Np          =   2000;
qN          =   400;
These depend on the parameter choices, where they can be relaxed far away
or above the resonance, while perhaps even pushed to larger values if the
resonance is very narrow and probing the high energy tail of the distribution

Version: 23-01-2021
Contact: Andrzej Hryczuk (NCBJ)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *)

(*********************** specification of the model ***************************)

Print["-----------------Model: Scalar Singlet DM ----------------"];

(* constants - fixed for the model *)
gDM                         =   1;  (* DM is a scalar *)
Nscatt                      =   2;  (* scattering on fermions and anti-fermions *)

cardinfo:={"-------- card: "<>parameters<>"  mDM="<>ToString[N[mDM]]<>
           "  ls="<>ToString[N[ls]]<>"  QCD="<>QCDcase<>" --------"};


(*************************** annihilation cross section ***********************)

(* position of the Higgs resonance in s *)
resonance                   :=  If[2mDM<mh,{mh^2},{}];
threshold                   :=  {};

Module[{svtab,svan,DhI,Dh,vS,vH,ar,ai,ySS,xSS,GHtmp},

(* since here sv_CMS a conversion factor of
v_lab/v_CMS                 =   s/(2s - 4mDM^2)
is applied to be in correct form to calculate <sv> *)
sv[s_]                      :=  s/(2s - 4mDM^2)(svtab[s] + Sum[svan[s,i], {i,{1,2,7,12}}]) //Re;
(*
if instead of using pretabulated Higgs width one used analytical formulae
(mostly tree-level!) one can use the expression below:
sv[s_]                      :=  s/(2s - 4mDM^2)( Sum[svan[s,i], {i,1,12}] +  Sum[svangg[s,i], {i,1,6}]) //Re;
*)

(* useful shorthand notation: *)
Gh                          =   0.0041102;
DhI[s_]                     :=  1/((s-mh^2)^2 + mh^2(GammaInv[mDM,ls]+Gh)^2);
Dh[s_]                      :=  1/((s-mh^2)^2 + mh^2 Gh^2);
vS[s_]                      :=  Sqrt[1-4mDM^2/s];
vH[s_]                      :=  Sqrt[1-4mh^2/s];
ar[s_]                      :=  1 + 3 mh^2 (s - mh^2) DhI[s];
ai[s_]                      :=  3 mh^2 Sqrt[s] (GammaInv[mDM,ls]+Gh) DhI[s];
ySS[s_]                     :=  ls v0^2/(s-2mh^2);
xSS[s_]                     :=  2 vS[s] vH[s]/(1+vH[s]^2);

(* where invisible contribution to Higgs width; if mDM<mh/2 *)
GammaInv[mDM_,ls_]          :=  GammaInv[mDM, ls] = HeavisideTheta[1-4 mDM^2/mh^2] (ls^2 v0^2)/(32 Pi mh) Sqrt[1-4 mDM^2/mh^2];

(*-------------------------------- tabulated ---------------------------------*)
svtab[s_]                   :=  (2 ls^2 v0^2)/Sqrt[s] GammaH[Sqrt[s]] DhI[s];

GHtmp                       =   Drop[Import["./models/ScalarSingletDM/Hwidth.dat", "Data"], 3];
GammaH                      =   Interpolation[SetPrecision[GHtmp,\[Infinity]],InterpolationOrder->1];

(*-------------------------- analytic expressions ----------------------------*)
(* annihilation to fermions *)
svan[s_,i_ /; i<=9]         :=  HeavisideTheta[1-4 m[ql[[i]]]^2/s] (m[ql[[i]]] ls)^2/(4 Pi) (1-4 m[ql[[i]]]^2/s)^(3/2) DhI[s] sf[s,i];
(* where the symmetry factor *)
sf[s_, i_ /; 9>=i>6]       :=  1; (* for leptons *)
sf[s_, i_ /; i<=6]         :=  If[3/2 Log[m[ql[[i]]]^2/s]+9/4<0,3,3(1+(4 alphaS)/(3 Pi) (3/2 Log[m[ql[[i]]]^2/s]+9/4))]; (* for quarks *)
(* annihilation to W and Z *)
svan[s_,10]                 :=  HeavisideTheta[1-4 m["W"]^2/s] (ls^2 s)/(8 Pi) Sqrt[1-4 m["W"]^2/s] DhI[s] (1-4 m["W"]^2/s + 12(m["W"]^2/s)^2);
svan[s_,11]                 :=  HeavisideTheta[1-4 m["Z"]^2/s] (ls^2 s)/(8 Pi) 1/2 Sqrt[1-4 m["Z"]^2/s] DhI[s] (1-4 m["Z"]^2/s + 12(m["Z"]^2/s)^2);
(* annihilation to hh *)
svan[s_,12]                 :=  ls^2 vH[s]/(16 Pi s) (ar[s]^2 + ai[s]^2 +8ySS[s]^2/(1-xSS[s]^2)- 2ySS[s](ar[s]-ySS[s])/xSS[s] Log[((1+xSS[s])/(1-xSS[s]))^2]);
(* annihilation to gg *)
svangg[s_,i_]               :=  2ls^2/s/8/Pi DhI[s] m[ql[[i]]]^4 (alphaS/Pi)^2 Abs[-2 - (1-4 m[ql[[i]]]^2/s)( PolyLog[2, -2/(Sqrt[1 - 4 m[ql[[i]]]^2/s] - 1)] + PolyLog[2, 2/(Sqrt[1 - 4 m[ql[[i]]]^2/s] + 1)])]^2;

(* --------------------------------- constants ------------------------------ *)
mh = 1251/10; (* Higgs mass *)
v0 = 246221/1000; (* vev *)
alphaS = 1172/10000; (* alpha strong (for QCD correction) *)

ql                          =   {"u", "d", "s", "c", "b", "t", "e", "mu", "tau"};
Vl                          =   {"W", "Z"}; Vdof = {3, 1};
m["u"]                      =   Rationalize[0.003];
m["d"]                      =   Rationalize[0.006];
m["s"]                      =   Rationalize[0.103];
m["c"]                      =   Rationalize[1.8743312820343210];
m["b"]                      =   Rationalize[4.8859454946067116];
m["t"]                      =   Rationalize[172.7];
m["e"]                      =   Rationalize[0.000511];
m["mu"]                     =   Rationalize[0.10566];
m["tau"]                    =   Rationalize[1.777];
m["W"]                      =   Rationalize[80.373281817049389];
m["Z"]                      =   Rationalize[91.1876];


(**************************  momentum transfer rate  **************************)

(* switch: "gamma(x)" or "M2" - if the latter, the gamma(x) is computed from
supplied amplitude squared  *)
scatttype                   =   "M2";

\[Alpha][m_, k_]                  :=
  Piecewise[{
  {(8 k^2 (-2 m^2 + 2 k^2 + mh^2))/(4 k^2 + mh^2)+2 (-2 m^2 + mh^2) Log[mh^2/(4 k^2 + mh^2)],k > 0.5}
  ,
  {k^8 ((768 m^2)/mh^8 - 128/mh^6) + k^6 (64/(3 mh^4) - (512 m^2)/(3 mh^6)) + (32 k^4 m^2)/mh^4, k <= 0.5}}
  ];

tM2f[k_, i_ /; i <= 6]     :=  3 2 m[ql[[i]]]^2 ls^2 \[Alpha][m[ql[[i]]], k];
tM2f[k_, i_ /; i > 6]      :=  2 m[ql[[i]]]^2 ls^2 \[Alpha][m[ql[[i]]], k];
tM2v[k_, i_]                :=  4 Vdof[[i]] 4 (m[Vl[[i]]]/2)^2 ls^2 ( -((4 k^2)/(4 k^2 + mh^2)) - Log[mh^2/(4 k^2 + mh^2)]);
kcm[k_, mDM_, i_]           :=  k mDM /Sqrt[mDM^2 + 2 om[m[ql[[i]]], k] mDM + m[ql[[i]]]^2];
om[m_, k_]                  :=  Sqrt[k^2 + m^2];(* energy-momentum dispersion relation *)

SetOptions[NIntegrate, WorkingPrecision -> $MachinePrecision, MaxRecursion -> 1000, MinRecursion -> 0, PrecisionGoal -> 4,
            Method -> {"GlobalAdaptive","MaxErrorIncreases"->10000, "SymbolicProcessing" -> 0.1}];

(* critical temperature in GeV *)
Tc                          =   0.154;

gamA[T_ /; T <= Tc ]        :=  1/(48 (Pi)^3 gDM mDM^3 T) Nscatt Check[NIntegrate[Sum[k/om[m[ql[[i]]], k] k^4  /(Exp[om[m[ql[[i]]], k]/T] +1) (1 - 1/(Exp[om[m[ql[[i]]], k]/T] + 1)) 1/(8 k^4)tM2f[kcm[k, mDM, i], i], {i, {7, 8, 9}}] +
      Sum[k/om[m[Vl[[i]]], k] k^4/(Exp[om[m[Vl[[i]]], k]/T] -1) (1 + 1/(Exp[om[m[Vl[[i]]], k]/T] - 1)) 1/(8 k^4)tM2v[k, i], {i, 1, Length[Vl]}], {k, T/100,T,100 T}],$MinMachineNumber] // Quiet;
gamA[T_ /; T > Tc ]         :=  1/(48 (Pi)^3 gDM mDM^3 T) Nscatt Check[NIntegrate[Sum[k/om[m[ql[[i]]], k] k^4  /(Exp[om[m[ql[[i]]], k]/T] +1) (1 - 1/(Exp[om[m[ql[[i]]], k]/T] + 1)) 1/(8 k^4)tM2f[kcm[k, mDM, i], i], {i, Length[ql]}] +
      Sum[k/om[m[Vl[[i]]], k] k^4/(Exp[om[m[Vl[[i]]], k]/T] -1) (1 + 1/(Exp[om[m[Vl[[i]]], k]/T] - 1)) 1/(8 k^4)tM2v[k, i], {i, 1, Length[Vl]}], {k, T/1000,T,1000 T}],$MinMachineNumber] // Quiet;

gamB[T_ /; T <= 4 Tc ]      :=  1/(48 (Pi)^3 gDM mDM^3 T) Nscatt NIntegrate[Sum[k/om[m[ql[[i]]], k] k^4  /(Exp[om[m[ql[[i]]], k]/T] +1) (1 - 1/(Exp[om[m[ql[[i]]], k]/T] + 1)) 1/(8 k^4)tM2f[kcm[k, mDM, i], i], {i, {7, 8, 9}}] +
      Sum[k/om[m[Vl[[i]]], k] k^4/(Exp[om[m[Vl[[i]]], k]/T] -1) (1 + 1/(Exp[om[m[Vl[[i]]], k]/T] - 1)) 1/(8 k^4)tM2v[k, i], {i, 1, Length[Vl]}], {k, T/100,T,100 T}]//Quiet ;
gamB[T_ /; T > 4 Tc ]       :=  1/(48 (Pi)^3 gDM mDM^3 T) Nscatt NIntegrate[Sum[k/om[m[ql[[i]]], k] k^4  /(Exp[om[m[ql[[i]]], k]/T] +1) (1 - 1/(Exp[om[m[ql[[i]]], k]/T] + 1)) 1/(8 k^4)tM2f[kcm[k, mDM, i], i], {i, {1, 2, 3, 7, 8, 9}}] +
      Sum[k/om[m[Vl[[i]]], k] k^4/(Exp[om[m[Vl[[i]]], k]/T] -1) (1 + 1/(Exp[om[m[Vl[[i]]], k]/T] - 1)) 1/(8 k^4)tM2v[k, i], {i, 1, Length[Vl]}], {k, T/100,T,100 T}]//Quiet ;


(* the final integration limit is taken such that \[Omega]>>T *)
gamma[x_]                   :=  If[QCDcase == "A",Abs[gamA[mDM/x]],Abs[gamB[mDM/x]]];
(* Abs[] added in case of a small numerical negative value *)
];
