(* %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Routine solving the Boltzmann equation on phase-space density level (fBE):

        E(\partial_t - Hp\partial_p)f_\[Chi](t,p) = C_ann + C_el

Input as global function:

        GetAnnMatrix[ ... ]             -  angularly averaged annihilation cross section < \[Sigma]v >_theta
        gam[x]                          -  momentum transfer rate \[Gamma] (Fokker-Planck approximation of C_el if FullCel = False in settings.wl)
        or GetScatMatrix[ ... ]         -  or full Cel (if FullCel=True in settings.wl)

Output:

        tfDM                            -   table containing normalized f_\[Chi] snapshots at xi, when swichting momentum coordinats, and xf.
        tYfBE                           -   table containing Y(x)
        tyfBE                           -   table containing y(x)
        Oh2fBE                          -   final value of relic abundance

Version: 15-01-2021
Contact: Tobias Binder (Kavli IPMU)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% *)

fBE:=Module[{xi,xf,h,hh,eps,errNewton,errMaxNewton,safety,error,KDo,AtoB,xip1,
             qNN,Nq,dq,tq,tq2,xq,fi,Yinit,feqi,feqip1,Fi,gammaDH,IM,threeIM,
             Derq1,Derq2,TrapEulFi,AM,SM,qTOp,gs,zeroM,gt,Ht,pref,firstmom,
             secondmom,entropy,xcap,tempt,xmintemp,tempOmega,AgainA,AgainB,End},

    xi                              =   xmin+0.;
    xf                              =   xmax+0.;
    h                               =   fBEhinit+0.;
    eps                             =   fBEerr+0.;
    errNewton                       =   fBEerrNewton+0.;
    errMaxNewton                    =   fBEerrMaxNewton+0.;
    safety                          =   fBEsafety+0.;
    KDo                             =   KDonly;
    AtoB                            =   qATOqB;

    (* Initialize *)

    qNN                             =   qN;
    Nq                              =   IntegerPart[qNN];
    dq                              =   (qmax-qmin)/(qNN-1.);
    tq                              =   Table[qmin+(k-1.)dq,{k,qNN}];
    tq2                             =   tq^2;
    Fi                              =   Table[0.,{k,qNN}];
    IM                              =   IdentityMatrix[Nq];
    zeroM                           =   IM-IM;
    threeIM                         =   3.*IM;
    Derq1                           =   GetDerivative[Nq,2,1]/dq;
    Derq2                           =   GetDerivative[Nq,2,2]/(dq^2);
    firstmom                        =   (gDM/(2 Pi^2))*mDM^3*dq*tq2;
    secondmom                       =   (gDM/(3*2*Pi^2))*mDM^5*dq*(tq2*tq2);

    (* If gam[xi]/(H[mDM/xi])>gamcap call nBE and set new xi with initial condition from nBE *)

    xcap                            =   Getxcap;

    If[(xcap==-1 && gam[xi]/(H[mDM/xi])>gamcap) || xcap==0,
        Print["fBE.wl: Error, root not found in interval or xcap>xmax."];Goto[End]];

    If[xcap==-1,
        xq                          =   Sqrt[xi^2+xi*tq2];
        If[!KDonly,fi=Exp[-xq],fi=(Exp[-xq]/Yeq[mDM,xi])]
        ,
        If[xi<=xcap<=xf,
            If[!KDonly,
                If[!nBEprevious,
                    xmax            =   xcap;
                    xmintemp        =   xmin;
                    xmin            =   0.9 xmin;
                    tempt           =   tYnBE;
                    tempOmega       =   Oh2nBE;
                    PrepANN;
                    nBE;
                    Yinit           =   Last[tYnBE][[2]];
                    xmax            =   xf;
                    xmin            =   xmintemp;
                    tYnBE           =   tempt;
                    Oh2nBE          =   tempOmega
                    ,
                    Yinit           =   Exp[Interpolation[Log[tYnBE],InterpolationOrder->1][Log[xcap]]];
                ]
                ,
                Yinit               =   1
            ];

            xi                      =   xcap+0.;
            xq                      =   Sqrt[xi^2+xi*tq2];
            fi                      =   Yinit*(Exp[-xq]/Yeq[mDM,xi]),
            Print["fBE.wl: error."];
            Goto[End]
        ];
    ];

    entropy                         =   s[mDM/xi];
    tfDM                            =   {{xi,tq2 fi/(tq2.fi)}};
    tYfBE                           =   {{xi,(1./(entropy*xi^(3/2)))(firstmom.fi)}}; (* xi, Y(xi) *)
    tyfBE                           =   {{xi,(1./(entropy^(5/3)*xi^(3/2)Last[tYfBE][[2]]))((secondmom/xq).fi)}}; (* xi, y(xi) *)

    (* %%%%%%%%%%%%%%%%%%%%%%%%%    fBE integration     %%%%%%%%%%%%%%%%%%%%%%%%%   *)

    While[Abs[(1-Last[tyfBE][[2]]/yeq[mDM,xi])]<AtoB,

        hh                          =   h;

        Label[AgainA];

            xip1                    =   xi+hh;

            If[xip1==xi,Print["fBE.wl: error, stepsize overflow, x = ",xi];Goto[End]];

            gt                      =   gtilde[mDM/xip1];
            Ht                      =   H[mDM/xip1]/(1+gt);
            pref                    =   (1./(2. Pi^2) mDM^3/(Ht xip1^(5./2.)))*dq;

            (* Annihilation options *)

            If[KDo,AM=zeroM,
                If[!Analyticsvtheta,
                    qTOp            =   mDM Sqrt[1/xip1];
                    AM              =   GetAnnMatrixTab[gDM*pref,Nq,qTOp,tq,tpregsvtheta,ltpregsvtheta,lpmin,lpmax](*Based on interpolated values*)
                    ,
                    AM              =   GetAnnMatrixAnalytic[gDM*pref,Nq,xip1,xip1,tq,modelparameters](*Based on analytic expressions *)
                ]
            ];

            (* Scattering options and Newton iteration (fBENewtonIt) *)

            If[!FullCel,
                (* Fokker-Planck (FP) approximation *)
                gammaDH             =   gam[xip1]/(2 Ht);
                TrapEulFi           =   fBENewtonItCFP[errNewton,errMaxNewton,Nq,1./2.+gt,xip1,hh,xip1,fi,tq,tq2,Fi,Derq1,Derq2,IM,threeIM,AM,gammaDH]
                ,
                (* full Cell *)
                SM                  =   GetScatMatrix[pref/(mDM^2),Nq,xip1,xip1,tq,modelparameters];
                TrapEulFi           =   fBENewtonItCel[errNewton,errMaxNewton,Nq,1./2.+gt,xip1,hh,xip1,fi,tq,tq2,Fi,Derq1,IM,AM,SM]
            ];

            (* TrapEulFi[[1]]=Trapeziodal fip1 , TrapEulFi[[2]]=Euler fip1 , TrapEulFi[[3]]= Fip1 *)

            error                   =   Max[Abs[(TrapEulFi[[1]]-TrapEulFi[[2]])/(TrapEulFi[[1]])]]/eps;

            If[error>1,hh=Max[safety/Sqrt[error],0.1]*hh;Goto[AgainA]];

        xi                          =   xip1;
        fi                          =   TrapEulFi[[1]];
        Fi                          =   TrapEulFi[[3]];
        xq                          =   Sqrt[xi^2+xi*tq2];
        entropy                     =   s[mDM/xi];
        tYfBE                       =   Append[tYfBE,{xi,(1./(entropy*xi^(3/2)))(firstmom.fi)}];
        tyfBE                       =   Append[tyfBE,{xi,(1./(entropy^(5/3)*xi^(3/2)Last[tYfBE][[2]]))((secondmom/xq).fi)}];

        (* uncomment (here and also below) if all the x points for fDM(x,q) are to ba stored: *)
        (*tfDM                      =   Append[tfDM,{xi,tq2 fi/(tq2.fi)}];*)

        If[error==0.,h=5.*hh,h=Min[safety/Sqrt[error],5.]*hh];

    ];

    tfDM                            =   Append[tfDM,{xi,tq2 fi/(tq2.fi)}];

    (* ------------------------Change of momentum coordinates A to B--------------------------*)

    gs                              =   iheff[mDM/xi];
    tq                              =   (Sqrt[xi]/gs^(1/3)) tq;
    tq2                             =   (xi/gs^(2/3)) tq2;
    dq                              =   (Sqrt[xi]/gs^(1/3))dq;
    Derq1                           =   (gs^(1/3)/Sqrt[xi]) Derq1;
    Derq2                           =   (gs^(2/3)/xi) Derq2;
    firstmom                        =   (gDM/(2 Pi^2))*mDM^3*dq*tq2;
    secondmom                       =   (gDM/(3*2*Pi^2))*mDM^5*dq*(tq2*tq2);
    xq                              =   Sqrt[xi^2+gs^(2/3)*tq2];
    feqi                            =   Exp[-xq];
    gt                              =   gtilde[mDM/xi];
    Ht                              =   H[mDM/xi]/(1+gt);
    gs                              =   iheff[mDM/xi];
    pref                            =   (1./(2. Pi^2) mDM^3/(Ht xi^(4)))*dq*gs;

    If[KDo,AM=zeroM,
        qTOp                        =   gs^(1/3)mDM/xi;
        If[!Analyticsvtheta,
            AM                      =   GetAnnMatrixTab[gDM*pref,Nq,qTOp,tq,tpregsvtheta,ltpregsvtheta,lpmin,lpmax]
            ,
            AM                      =   GetAnnMatrixAnalytic[gDM*pref,Nq,gs^(2/3),xi,tq,modelparameters]
        ]
    ];

    If[!FullCel,
        Fi                          =   (gam[xi]/(2 Ht))(1./xi)((1./(gs^(2/3)))xq(Derq2)+(tq+2.*xq/((gs^(2/3))tq)+ tq/xq)*(Derq1)+threeIM).fi+(((AM.(tq2 feqi))*feqi)-((AM.(tq2 fi))*fi))
        ,
        SM                          =   GetScatMatrix[pref/(mDM^2),Nq,gs^(2/3),xi,tq,modelparameters];
        Fi                          =   SM.fi+(((AM.(tq2 feqi))*feqi)-((AM.(tq2 fi))*fi))
    ];

    (* ---------------------- Continue with momentum coordinates B --------------------------*)

    While[xi<xf,

        hh                          =   h;

        If[xi+hh>xf,hh=xf-xi];

        Label[AgainB];

            xip1                    =   xi+hh;

            If[xip1==xi,Print["fBE.wl: error, stepsize overflow"];Goto[End]];

            gt                      =   gtilde[mDM/xip1];
            Ht                      =   H[mDM/xip1]/(1+gt);
            gs                      =   iheff[mDM/xip1];
            pref                    =   (1./(2. Pi^2) mDM^3/(Ht xip1^(4)))*dq*gs;

            If[KDo,AM=zeroM,
                If[!Analyticsvtheta,
                    qTOp            =   gs^(1/3)mDM/xi;
                    AM              =   GetAnnMatrixTab[gDM*pref,Nq,qTOp,tq,tpregsvtheta,ltpregsvtheta,lpmin,lpmax]
                    ,
                    AM              =   GetAnnMatrixAnalytic[gDM*pref,Nq,gs^(2/3),xi,tq,modelparameters]
                ]
            ];

            If[!FullCel,
                gammaDH             =   gam[xip1]/(2 Ht);
                If[gammaDH<minnumber,gammaDH=0.];
                TrapEulFi           =   fBENewtonItCFP[errNewton,errMaxNewton,Nq,0.,gs^(2/3),hh,xip1,fi,tq,tq2,Fi,Derq1,Derq2,IM,threeIM,AM,gammaDH]
                ,
                SM                  =   GetScatMatrix[pref/(mDM^2),Nq,gs^(2/3),xip1,tq,modelparameters];
                TrapEulFi           =   fBENewtonItCel[errNewton,errMaxNewton,Nq,0.,gs^(2/3),hh,xip1,fi,tq,tq2,Fi,Derq1,IM,AM,SM];
            ];

            error                   =   Max[Abs[(TrapEulFi[[1]]-TrapEulFi[[2]])/(TrapEulFi[[1]])]]/eps;

            If[error>1, hh=Max[safety/Sqrt[error],0.1]*hh; Goto[AgainB]];

        xi                          =   xip1;
        fi                          =   TrapEulFi[[1]];
        Fi                          =   TrapEulFi[[3]];
        h                           =   hh;
        xq                          =   Sqrt[xi^2+gs^(2/3)*tq2];
        entropy                     =   s[mDM/xi];
        tYfBE                       =   Append[tYfBE,{xi,(gs/(entropy*xi^(3)))(firstmom.fi)}];
        tyfBE                       =   Append[tyfBE,{xi,(gs^(5/3)/(entropy^(5/3)*xi^(4) Last[tYfBE][[2]]))((secondmom/xq).fi)}];

        (* uncomment if all the x points for fDM(x,q) are to ba stored: *)
        (*tfDM                        =   Append[tfDM,{xi,tq2 fi/(tq2.fi)}];*)

        If[error==0.,h=5.*hh,h=Min[safety/Sqrt[error],5.]*hh];
    ];

    Label[End];

    tfDM                            =   Append[tfDM,{xi, tq2 fi/(tq2.fi)}];
    Oh2fBE                          =   mDM*Entropytoday/Rhocritical Last[tYfBE][[2]];

    (* result for the relic density *)
    "Oh2fBE = " <> ToString[Oh2fBE]
];
